#!/usr/bin/env python3
import common
import json
from xml.dom import minidom
import math
import time
import codecs

def getVmMetrics(input):
    logger = common.getLogger()
    doc = minidom.parse(common.getspecialConfigValue("XmlTemplate",name = 'globalConfig'))
    logger.info("start create SAP HANA monitor xml")
    try:
        logger.debug("input is %s"%(input))
        datapointList = json.loads(input)['datapoints']
        listLength = len(datapointList)
        if(0 == listLength):
            logger.error("Get empty data from ces")
        else:
            inputDict = datapointList[listLength - 1]['value']
            exec("inputDict=" + inputDict)
            common.prepareDictData(inputDict)
            logger.debug("inputDict is %s"%(inputDict))
            createMetrics(doc, inputDict)
    except Exception,e:
        logger.error('create inputDict from input failed')
        logger.error(e)
    with codecs.open(common.getspecialConfigValue("TempXml",name = 'globalConfig'), 'wb') as out:
        doc.writexml(out, encoding='utf-8')

def dealCommonMetrics(docElement, inputDict):
    logger = common.getLogger()
    try:
        docElementName = docElement.getElementsByTagName("name")[0].childNodes[0].nodeValue
        elementRefName = common.getspecialConfigValue(docElementName,name = 'xmlConfig')
        if("undo" == elementRefName):
            logger.info("Element %s does't support"%(docElementName))
        #cloud provider is from /sys/class/dmi/id/chassis_asset_tag
        elif("undefine" == elementRefName):
            if("Data Provider Version" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = common.getspecialConfigValue("DataProviderVersion",name = 'globalConfig')
            elif("Cloud Provider" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = common.getCloudProvider()
            elif("CPU Over-Provisioning" == docElementName):
                cpuOverProvisioning = 'yes'
                if(1 == int(inputDict.get("numanodestrategy")) or int(int(common.getCpuCoreNum()) * float(inputDict.get("max_hw_frequency"))) == int(float(inputDict.get("cpu_reservation"))) or 1 == int(inputDict.get("cpu_ratio"))):
                    cpuOverProvisioning = 'no'
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = cpuOverProvisioning
            elif("Guaranteed VM Processing Power" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = common.getCpuCoreNum()
            elif("Current VM Processing Power" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = common.getCpuCoreNum()
            elif("Max. VM Processing Power" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = common.getCpuCoreNum()
            elif("Guaranteed Memory assigned" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(common.getMemoryTotal())
            elif("Current Memory assigned" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(common.getMemoryTotal())
            elif("Max Memory assigned" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(common.getMemoryTotal())
            elif("Memory Consumption" == docElementName):
                docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = ("%0.2f" % float(common.getMemoryConsumption()))
            else:
                logger.info("Element %s does't support"%(docElementName))
            
        else:
            docElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = inputDict.get(elementRefName)
    except Exception,e:
        logger.error('create common from input failed')
        logger.error(e)

def addDiskMetrics(doc, docElement, inputDict):
    logger = common.getLogger()
    disks = inputDict.get("volumes")
    diskNum = 0
    for disk in disks:
        diskNum += 1
        #use deepcopy can not success copy.deepcopy(docElement)
        try:
            diskDocElement = docElement.cloneNode(True)
            # vmDiskName = ''
            diskDocElement.setAttribute("device-id",disk.get("volume_name"))
            docElementName = diskDocElement.getElementsByTagName("name")[0].childNodes[0].nodeValue
            if("Volume Type" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = disk.get("volume_type")
            elif("Volume ID" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = disk.get("volume_name")
            elif("Mapping" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = \
                common.get_device_name(disk.get("volume_wwn"), disk.get("volume_identify"))
            elif("Volume Read Response Time" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(math.ceil(float(disk.get("rd_ticks"))))
            elif("Volume Write Response Time" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(math.ceil(float(disk.get("wr_ticks"))))
            elif("Volume Utilization" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = ("%0.2f" % float(disk.get("tot_ticks")))
            elif("Volume Queue Length" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(math.ceil(float(disk.get("rq_tick"))))
            elif("Volume Read Throughput" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(float(disk.get("byte_out"))*1024)
            elif("Volume Write Throughput" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(float(disk.get("byte_in"))*1024)
            elif("Volume Read Ops" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(math.ceil(float(disk.get("rd_ios"))))
            elif("Volume Write Ops" == docElementName):
                diskDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(math.ceil(float(disk.get("wr_ios"))))
            else:
                logger.info('Disk element %s is not supported'%(docElementName))
        except Exception,e:
            logger.error('create disk from input failed')
            logger.error(e)
            logger.error(docElement.toxml())
        doc.documentElement.insertBefore(diskDocElement, docElement)
        text = doc.createTextNode('\n\n  ')
        if diskNum != len(disks):
            doc.documentElement.insertBefore(text, docElement)
    doc.documentElement.removeChild(docElement)

    
def addNicMetrics(doc, docElement, inputDict):
    logger = common.getLogger()
    nics = inputDict.get("nics")
    nicNum = 0
    for nic in nics:
        nicNum += 1
        #nicDocElement = copy.deepcopy(docElement)
        try:
            nicDocElement = docElement.cloneNode(True) #copyDocumentElement(doc, docElement)
            mac_address = nic.get("mac_address")
            if not common.isValidMac(mac_address):
                logger.error('mac_address %s is invalid'%(mac_address))
                continue
            nicName = common.getNicByMac(mac_address)
            nicDocElement.setAttribute("device-id",nic.get("nic_name"))
            docElementName = nicDocElement.getElementsByTagName("name")[0].childNodes[0].nodeValue
            if("Adapter ID" == docElementName):
                nicDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = nic.get("nic_name")
            elif("Mapping" == docElementName):
                nicDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = nicName
            elif("Minimum Network Bandwidth" == docElementName):
                nicDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(nic.get("current_speed"))
            elif("Current Network Bandwidth" == docElementName):
                nicDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(nic.get("current_speed"))
            elif("Maximum Network Bandwidth" == docElementName):
                nicDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = int(nic.get("current_speed"))
            elif("Health Indicator" == docElementName):
                if("UP" == nic.get("status")):
                    network_health = ((float(nic.get("byte_in")) + float(nic.get("byte_out")))/1024) / int(nic.get("current_speed"))
                    nicDocElement.getElementsByTagName("value")[0].childNodes[0].nodeValue = round(network_health,2)
                else:
                    logger.error('nic status is error,the value is %s'%(nic.get("status")))
            else:
                logger.error('Disk element %s does\'t support'%(docElementName))
        except Exception,e:
            logger.error('create nic from input failed')
            logger.error(e)
            logger.error(nicDocElement.toxml())
        doc.documentElement.insertBefore(nicDocElement, docElement)
        text = doc.createTextNode('\n\n  ')
        if nicNum != len(nics):
            doc.documentElement.insertBefore(text, docElement)
    doc.documentElement.removeChild(docElement)
    
def createMetrics(doc, inputDict):
    logger = common.getLogger()
    try:
        metrics = doc.documentElement.getElementsByTagName(common.getspecialConfigValue("XmlTagName",name = 'globalConfig'))
        timeNow = str(time.time()).split('.')[0]
        for metric in metrics:
            metric.setAttribute("last-refresh", timeNow)
            category = metric.getAttribute("category")
            if("disk" == category):
                #add disk metrics
                addDiskMetrics(doc, metric, inputDict)
            elif("network" == category):
                addNicMetrics(doc, metric, inputDict)
                #add nic metrics
            else:
                dealCommonMetrics(metric, inputDict)
        logger.info("create the xml complete")
    except Exception,e:
        logger.error('create SAP HANA monitor xml error')
        logger.error(e)

