#!/usr/bin/env python3
# -*- coding:utf-8 -*-

#import binascii
import datetime
import json
import multiprocessing
import os
import time

import common
import collection
import rest_tool
from web_service import start_server


class MonitorMetricsThread(multiprocessing.Process):
    def __init__(self, interval, args=()):
        multiprocessing.Process.__init__(self, args=())
        self.interval = interval
        self.akskShareArray = args[0]

    def __isNeedFreshAKSK(self, akskStr):
        isNeed = False
        if len(akskStr) == 0:
            isNeed = True
        else:
            shareAKSKJson = json.loads(akskStr)
            expiredTime = shareAKSKJson.get("expires_at")
            expiredTimeFormat = datetime.datetime.strptime(expiredTime,"%Y-%m-%dT%H:%M:%S.%fZ")
            curTimeFormat = datetime.datetime.utcnow()
            # if aksk will expired in 5 minutes, need to reget aksk
            if curTimeFormat + datetime.timedelta(minutes=5) >= expiredTimeFormat:
                isNeed = True
        return isNeed
        
    def __isOpenStackAKSKJsonValid(self, aksk):
        """Check aksk json format.
        {
            "expires_at": "2015-02-27T18:30:59.999999Z",
            "access": "xxxXXXxxxxxx",
            "secret": "xxxXXXXXXXXXXXX",
            "securitytoken": "xxxXXXXXXXXXXXX"
            }
        """
        legalAKSKJsonKey = ["expires_at", "access", "secret", "securitytoken"]
        jsonKeys = aksk.keys()
        for key in legalAKSKJsonKey:
            if key not in jsonKeys:
                return False
        jsonValues = aksk.values()
        for value in jsonValues:
            if len(value) == 0:
                return False
        return True
    
    
        
    def __getAKSKFromOpenstack(self, akskArray):
        akskStr = akskArray.value
        if self.__isNeedFreshAKSK(akskStr) is not True:
            return json.loads(akskStr)
  
        logger.info("AKSK will expired in 5 min, begin to get new AKSK")
        tryTimes = 3
        tmpJsonString = None
        while tryTimes > 0:
            tmpJsonString = common.getAKSKFromOpenStack()
            if tmpJsonString is not None:
                break
            else:
                time.sleep(10)
                tryTimes -= 1
        if tmpJsonString is None:
            logger.error('Failed to get AKSK from OpenStack')
            return None
        
        try:
            akskJsonDic = json.loads(tmpJsonString)["credential"]
        except Exception,e:
            logger.error('Failed to loads String to Json')
            logger.error(e)
            return None
        
        if self.__isOpenStackAKSKJsonValid(akskJsonDic) is True:
            akskArray.value = json.dumps(akskJsonDic)
            logger.debug('Succeeded to get AKSK from OpenStack')
            return akskJsonDic
        else:
            logger.error('AKSK Json reponse is invalid')
            return None
                    
    def run(self):
        logger.info("Start to get VM monitor metrics")
        while True:
            akskJsonDic = self.__getAKSKFromOpenstack(self.akskShareArray)
            if akskJsonDic is None:
                time.sleep(self.interval)
                continue
            ###monitor to get data
            try:
                host = common.getspecialConfigValue("CesDomainName",name = 'globalConfig')
                tenant_id = common.getOpenstackValue("project_id")
                server_id = common.getOpenstackValue("uuid")
                access_key = akskJsonDic["access"]
                secret_key = akskJsonDic["secret"]
                secret_token = akskJsonDic["securitytoken"]
                retCode,response = rest_tool.getMetricsFromCes(host, tenant_id, server_id, access_key, secret_key, secret_token,
                                                              int(common.getspecialConfigValue("RestTimeOut",name = 'globalConfig')))
                if(200 != retCode):
                    logger.error('getMetricsFromCes has error,code is %s,response is %s'%(retCode, response))
                else:
                    collection.getVmMetrics(response)
            except Exception,e:
                logger.error('getMetricsFromCes failed')
                logger.error(e)
            time.sleep(self.interval)

class WebServiceThread(multiprocessing.Process):
    def __init__(self, interval):
        multiprocessing.Process.__init__(self)
        self.interval = interval
    
    def run(self):
        port = common.getspecialConfigValue("WebServicePort",name = 'globalConfig')
        logger.info("start the webService.port is %s"%(port))
        start_server(port)
        ###webservice
        time.sleep(self.interval)
            
class AuditServiceThread(multiprocessing.Process):
    def __init__(self, interval, args=()):
        multiprocessing.Process.__init__(self, args=())
        self.interval = interval
        self.akskShareArray = args[0]
    
    def __getAKSKFromShareArray(self, akskString):
        if len(akskString) == 0:
            return None
        return json.loads(akskString)
        
    def run(self):
        logger.info("Start to audit VM monitor metrics")
        while True:
            ###audit
            akskJsonDic = self.__getAKSKFromShareArray(self.akskShareArray.value)
            if akskJsonDic is None:
                logger.info("No valid AKSK, wait...")
                time.sleep(15)
                continue
            count = 0
            while count < 3:
                try:
                    host = common.getspecialConfigValue("EcsDomainName",name = 'globalConfig')
                    tenant_id = common.getOpenstackValue("project_id")
                    server_id = common.getOpenstackValue("uuid")
                    access_key = akskJsonDic["access"]
                    secret_key = akskJsonDic["secret"]
                    secret_token = akskJsonDic["securitytoken"]
                    (retCode,response) = rest_tool.registVm(host, tenant_id, server_id, access_key, secret_key, secret_token,
                                                           int(common.getspecialConfigValue("RestTimeOut",name = 'globalConfig')))
                    if(200 == retCode):
                        count = 3
                        logger.info("registVm success!")
                    else:
                        count = count + 1
                        logger.error('registVm has error:%s'%(response))
                except Exception,e:
                    count = count + 1
                    logger.error('registVm failed')
                    logger.error(e)
                time.sleep(60)
            count = 0
            time.sleep(self.interval)
            
if __name__ == '__main__':
    
    AKSK_STR_LEN = 17000 # the length of AKSK json string 
    
    common.instanceLogger()
    logger = common.getLogger()
    #load SpecialCongifig
    common.instanceSpecialConfigJson(logger)
    #check thread is Run
    isExist = common.isExistThreadRun(os.getpid(), common.getspecialConfigValue("MainScript",name = 'globalConfig'))
    if isExist:
        logger.error("The old instance %s is running"%(isExist))
        exit()
    #load metadata
    tryTimes = 0
    while tryTimes < 60:
        if(common.instanceOpenstackJson()):
            logger.info("load metadata complete")
            tryTimes = 60
        else:
            tryTimes = tryTimes +1
            logger.error("Get metadata failed,tryTimes:%s"%(tryTimes))
            if tryTimes >= 60:
                logger.fatal("Failed to get metadata for 60 times, exit sapmonitor.")
                exit()
            else:
                time.sleep(60)
    logger.info("instance information complete")
    # create a share dict to store aksk json dictionary.
    akskStr = multiprocessing.Array('c', AKSK_STR_LEN)
    monitorInterval = int(common.getspecialConfigValue("MonitorInterval",name = 'globalConfig'))
    monitorMetricsThread = MonitorMetricsThread(monitorInterval, args=(akskStr,))
    webServiceThread = WebServiceThread(1)

    auditInterval = int(common.getspecialConfigValue("AuditInterval",name = 'globalConfig'))
    auditServiceThread = AuditServiceThread(auditInterval, args=(akskStr,))
    monitorMetricsThread.start()
    webServiceThread.start()
    auditServiceThread.start()
    
    logger.info("Start service ok")
