#!/bin/sh
initpath="/etc/init.d"
basepath=$(cd `dirname $0`; pwd)
servicename=hwdataprovider
mainpython="dataprovider.py"
serviceupdatename=hwdataproviderupdate
updatemainpython="dataprovider_update.py"
guard="guard.py"
userhomedir=/home/hwdataprovider
username=hwdataprovider
logdir=/var/log/huawei/dataprovider
configFile="$basepath/config.json $basepath/logger.conf $basepath/metrics_template.xml"
groupadd $username

useradd -d $userhomedir -c "dataprovider user" -m $username -g  $username -s /sbin/nologin
chown $username:$username $userhomedir

touch /etc/sudoers.d/90-$servicename
chmod 440 /etc/sudoers.d/90-$servicename
sed -i "/^hwdataprovider ALL=(root) NOPASSWD:/d" /etc/sudoers.d/90-$servicename
fdiskDir=`which fdisk`
ifconfigDir=`which ifconfig`
echo "hwdataprovider ALL=(root) NOPASSWD:$basepath/install.sh,$fdiskDir -l,$ifconfigDir" >> /etc/sudoers.d/90-$servicename

sed -i "s@sudo ifconfig@sudo $ifconfigDir @" $basepath/common.py

mkdir -p $logdir
chown $username:$username -R $logdir
chmod 755 $logdir

mkdir -p $basepath/tmp


function version_ge()
{
    test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" == "$1";
}

#obtain params and storage python execute path
golbePythonPath="python"    #default is python(/usr/bin/python)
if [ $# -eq 1 ]             #param = 1
then
    param="$1"
    echo "python execute path is -- $param"
    python_version=`"$param" -V 2>&1 |grep -i python| awk -F " " '{print $2}'`
    echo ${python_version}
    if [ "x${python_version}" == "x" ]
    then
        echo "python version number empty"
    elif version_ge $python_version 2.7.5
    then
        echo "python version  is correct"    #version  compliance
        golbePythonPath=`echo $param`  #python new execute path
        sed -i "s@\"PythonPath\":\"\"@\"PythonPath\":\"$golbePythonPath\"@" $basepath/config.json
    else
        echo "python version should be 2.7.5 or above"
        echo "python version is incorrect"
        exit
    fi
elif [ $# -eq 0 ] #param is empty
then
    echo "use default python"
    python_version=$(python -V 2>&1 |grep -i python| awk -F " " '{print $2}')
    if [ "x${python_version}" == "x" ]
    then
        echo "python version number empty"
    elif version_ge $python_version 2.7.5
    then
        echo "python version is correct"    #version  compliance
    else
        echo "python version should be 2.7.5 or above"
        echo "python version is incorrect"
        exit
    fi
else
    echo "The number of input parameters is incorrect"
    exit      
fi

echo "#!/bin/bash
# /etc/init.d/$servicename
### BEGIN INIT INFO
# Provides: $servicename
# Required-Start: \$remote_fs
# Required-Stop: \$remote_fs
# Default-Start: 2 3 5
# Default-Stop: 0 1 6
# Short-Description: dataprovider initscript
# Description: This service is used to provider metrics for sap
### END INIT INFO

case \"\$1\" in
    start)
        echo \"Starting $servicename\"
        su -s /bin/bash - $username -c \"cd $basepath;$golbePythonPath $basepath/$mainpython  >/dev/null 2>&1 &\"
        ;;
    stop)
        echo \"Stopping $servicename\"
        ps aux | grep python |grep -F \"$basepath/$mainpython\" |grep -v grep| cut -c 9-15 | xargs kill
        ;;
    restart)
        \$0 stop
        \$0 start
        ;;
    force-reload)
        echo \"Reload service $servicename\"
        checkproc $servicename
        ;;
    status)
        pid=\`ps aux | grep python | grep \"$basepath/$mainpython\" |grep -v grep| cut -c 9-15 \`
        if [ -n \"\$pid\" ]
        then
            echo \"$servicename status:                                         running\"
        else
            echo \"$servicename status:                                        unuse\"
        fi
        ;;
    *)
        echo \"Usage: service $servicename start|stop|restart|force-reload\"
        exit 1
        ;;
esac
exit 0" >$initpath/$servicename

chmod 750 $initpath/$servicename
chmod 750 $userhomedir
chmod 750 $basepath/*
chmod 640 $configFile 
chmod 750 $basepath/tmp
chown $username:$username -R $basepath
cd $initpath
chkconfig --add $servicename >/dev/null 2>&1
$initpath/$servicename stop >/dev/null 2>&1
$initpath/$servicename start

echo "#!/bin/bash
# /etc/init.d/$serviceupdatename
### BEGIN INIT INFO
# Provides: $serviceupdatename
# Required-Start: \$remote_fs
# Required-Stop: \$remote_fs
# Default-Start: 2 3 5
# Default-Stop: 0 1 6
# Short-Description: dataprovider update initscript
# Description:
### END INIT INFO

case \"\$1\" in
    start)
        echo \"Starting $serviceupdatename\"
        su -s /bin/bash - $username -c \"cd $basepath;$golbePythonPath $basepath/$updatemainpython >/dev/null 2>&1 &\"
        ;;
    stop)
        echo \"Stopping $serviceupdatename\"
        ps aux | grep  python | grep \"$basepath/$updatemainpython\" |grep -v grep| cut -c 9-15 | xargs kill
        ;;
    restart)
        \$0 stop
        \$0 start
        ;;
    force-reload)
        echo -n \"Reload service $serviceupdatename\"
        checkproc $serviceupdatename
        ;;
    status)
        pid=\`ps aux | grep python | grep \"$basepath/$updatemainpython\" |grep -v grep| cut -c 9-15 \`
        if [ -n \"\$pid\" ]
        then
            echo \"$serviceupdatename status:                                         running\"
        else
            echo \"$serviceupdatename status:                                        unuse\"
        fi
        ;;
    *)
        echo \"Usage: service $serviceupdatename start|stop|restart|force-reload\"
        exit 1
        ;;
esac
exit 0" >$initpath/$serviceupdatename

chmod 750 $initpath/$serviceupdatename
cd $initpath
chkconfig --add $serviceupdatename >/dev/null 2>&1
$initpath/$serviceupdatename stop >/dev/null 2>&1
$initpath/$serviceupdatename start

#start crontab service
touch /etc/cron.d/90-$servicename
sed -i "/guard.py$/d" /etc/cron.d/90-$servicename
echo "1 */6 * * *    root $golbePythonPath $basepath/$guard" >> /etc/cron.d/90-$servicename
